package net.lapisblue.server.images;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.text.*;
import java.io.*;
import java.util.*;
import javax.imageio.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class EdgeServlet extends ImageServlet {
  ShapeServlet shapeServlet = new ShapeServlet();
  
  class ShapeGenerator extends ImageGenerator {
    final int width;
    final int height;
    final float stroke;
    final Color color;
    final Color fill;
    final String type;
    
    ShapeGenerator (HttpServletRequest request) {
      super(request);

      this.stroke = getFloat("s");
      this.width = getInt("w");
      this.height = getInt("h");
      this.color = getColor("c");
      this.fill = getColor("f");
      this.type = getString("t");
    }
    
    protected void draw (Graphics2D g2d) {
      g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                           RenderingHints.VALUE_ANTIALIAS_ON);

      final Shape shape;
      
      if ("round".equals(type)) {
        final float corner = getFloat("corner");

        shape = new RoundRectangle2D.Float(stroke*0.5f, stroke*0.5f, 
                                           width-stroke-1, height-stroke-1,
                                           2*corner, 2*corner);
        
      } else if ("tri".equals(type)) {
        final float x0 = getFloat("x0");
        final float x1 = getFloat("x1");
        final float x2 = getFloat("x2");
        final float y0 = getFloat("y0");
        final float y1 = getFloat("y1");
        final float y2 = getFloat("y2");

        final GeneralPath path = new GeneralPath();
        
        path.moveTo(x0 * width, y0 * height);
        path.lineTo(x1 * width, y1 * height);
        path.lineTo(x2 * width, y2 * height);
        path.closePath();
        
        shape = path;
      } else if ("star".equals(type)) {
        final int points = Math.min(Math.max(getInt("points"), 2), 200);
        final double rotation = Math.toRadians(getDouble("rotate"));
        final double r1 = getDouble("r1");
        final double r2 = getDouble("r2");
        final GeneralPath path = new GeneralPath();
        
        double theta = rotation;
        final double delta = Math.PI / points;
        
        double x;
        double y;
        
        x = 0.5 + 0.5 * r1 * Math.sin(theta);
        y = 0.5 + 0.5 * r1 * Math.cos(theta);

        path.moveTo((float)(x * width), (float)(y * height));

        for (int point = 0; point < points; point ++) {
          theta += delta;

          x = 0.5 + 0.5 * r2 * Math.sin(theta);
          y = 0.5 + 0.5 * r2 * Math.cos(theta);
        
          path.lineTo((float)(x * width), (float)(y * height));

          theta += delta;
        
          x = 0.5 + 0.5 * r1 * Math.sin(theta);
          y = 0.5 + 0.5 * r1 * Math.cos(theta);
        
          path.lineTo((float)(x * width), (float)(y * height));
        }

        path.closePath();
        
        shape = path;
      } else if ("poly".equals(type)) {
        final int vertices = Math.min(Math.max(getInt("vertices"), 3), 200);
        final double rotation = Math.toRadians(getDouble("rotate"));
        final double r = getDouble("r");
        final GeneralPath path = new GeneralPath();
        
        double theta = Math.PI + rotation;
        final double delta = 2 * Math.PI / vertices;
        
        double x;
        double y;
        
        x = 0.5 + 0.5 * r * Math.sin(theta);
        y = 0.5 + 0.5 * r * Math.cos(theta);

        path.moveTo((float)(x * width), (float)(y * height));

        for (int vertex = 0; vertex < vertices; vertex ++) {
          theta += delta;

          x = 0.5 + 0.5 * r * Math.sin(theta);
          y = 0.5 + 0.5 * r * Math.cos(theta);
        
          path.lineTo((float)(x * width), (float)(y * height));
        }

        path.closePath();
        
        shape = path;
      } else if ("ellipse".equals(type)) {
        final float radius = getFloat("r");
        shape = new Ellipse2D.Float(width*0.5f * (1-radius), height*0.5f * (1-radius),
                                    width*0.5f * (2*radius), height*0.5f * (2*radius));
      } else {
        shape = new Rectangle2D.Float(stroke*0.5f, stroke*0.5f, 
                                      width-stroke-1, height-stroke-1);
      }
      
      final Color fill;
      final Color color;
      
      if ((this.fill == null) && 
          (this.color == null)) {
        fill = Color.white;
        color = Color.black;
      } else {
        fill = this.fill;
        color = this.color;
      }
      
      if (fill != null) {
        g2d.setColor(fill);
        g2d.fill(shape);
      }
      
      if (color != null) {
        g2d.setColor(color);
        g2d.setStroke(new BasicStroke(stroke));
        g2d.draw(shape);;
      }
    }
    
    protected int getImageWidth () { return width; }
    protected int getImageHeight () { return height; }
  }
 
  protected ImageGenerator createImageGenerator (HttpServletRequest request) {
    return new ShapeGenerator(request);
  }
}